const UNQUOTED = 0;
const QUOTED = 1;
const ESCAPED = 2;

const STATE_COUNT = 3;

class MonoidElement {
  constructor(states) {
    this.states = states;
  }

  get valid() {
    return this.states[UNQUOTED] === UNQUOTED;
  }
}

export const IDENTITY_ELEMENT = new MonoidElement([
  UNQUOTED,
  QUOTED,
  ESCAPED,
]);

const QUOTE = new MonoidElement([
  QUOTED,
  UNQUOTED,
  QUOTED,
]);

const BACKSLASH = new MonoidElement([
  UNQUOTED,
  ESCAPED,
  QUOTED,
]);

const OTHER = new MonoidElement([
  UNQUOTED,
  QUOTED,
  QUOTED,
]);

export function encodeAsMonoidElement(character) {
  switch (character) {
  case '"':
    return QUOTE;
  case '\\':
    return BACKSLASH;
  default:
    return OTHER;
  }
}

export function combineMonoidElements(left, right) {
  const states = [];
  for (let i = 0; i < STATE_COUNT; ++i) {
    states.push(right.states[left.states[i]]);
  }
  return new MonoidElement(states);
}
