// Extra imports for working around a V8 bug; see below.
import { SourceTextModule } from 'vm';
import { fileURLToPath } from 'url';
import { resolve, dirname } from 'path';
import { readFileSync } from 'fs';

import { jest } from '@jest/globals';

// This is included for consistency with `create-react-app`.  It may be become
// unnecessary in future versions of Jest (see
// https://github.com/facebook/jest/issues/9430#issuecomment-616232029), but
// Jest may also choose to move away from magic globals and use imports instead,
// which are already opt-in (see https://github.com/facebook/jest/issues/4473).
globalThis.jest = jest;

// Work around <https://bugs.chromium.org/p/chromium/issues/detail?id=1244145>,
// a rather nasty bug in V8 where source code dynamically imported with
// host-defined options (see, e.g., the `options` parameter to
// <https://nodejs.org/api/vm.html#class-vmsourcetextmodule>) will be cached
// with only its source text as key so that later imports with different options
// are incorrectly treated as cache hits.
//
// Since Jest uses different options (see, e.g., `loadEsmModule` in
// `…/node_modules/jest-runtime/build/index.js`) for each test file, that means
// that while processing any test file after the first, `mapReduce.js` is still
// associated with an old test environment, and any attempts by it to
// dynamically import a monoid go through an environment that has been torn
// down.  They therefore fail.
//
// Since `mapReduce.js` is unmocked and code under test, we don't need any of
// Jest's special import handling (e.g., support for mocking) in it or any of
// its imports, so here we preload `mapReduce.js` into V8's cache without any
// host-defined options.  There are still false cache hits, but at least none of
// the cache entries are associated with a test environment.
//
// eslint-disable-next-line no-new
new SourceTextModule(readFileSync(resolve(dirname(fileURLToPath(import.meta.url)), 'mapReduce.js'), 'utf8'));
